/*
 * ConectivityMatrix.h
 *
 *  Created on: 22.10.2009
 *      Author: hartman
 */

#ifndef CONECTIVITYMATRIX_H_
#define CONECTIVITYMATRIX_H_
#include "Matrix.h"
#include <string>
#include <list>
#include <vector>

using namespace std;

typedef unsigned long ULLONG_SUM_TYPE;
typedef long LLONG_SUM_TYPE;

/*! Values for no degree using in average degree degree */
#define NO_DEGREE -1


class Graph {
public:
	/*!
	 *	Constructor of class representing symmetric matrix of unweighted connections from existing data.
	 * @param mat The 0/1 upper triangular matrix
	 */
	Graph(SymMatrix<bool> *mat);

	/**with distance matrix only, data can be NULL */
	Graph(SymMatrix<bool> *mat, SymMatrix<int> *dmat);

	/*!
	 * Destructor
	 */
	virtual ~Graph();

	/** Get value from given position
	 * \note{Validated}
	 *
	 * 	@param i row specification
	 * 	@param j column specification
	 *  @return value from position i,j
	 * */
	char operator() (int i, int j);

	/*!
	 * Get size of matrix which means number of its rows.
	 *
	 * \note{Validated}
	 */
	int getSize();

	/*!
	 * 	Method to get number of edges of ghraph
	 * \note{Validated}
	 *
	 * @return Number of edges.
	 */
	double getNumberOfEdges();

	/*!
	 * Get number of triangles containing specific node.
	 * \note{Validated}
	 *
	 * @param nodeIndex Index of node for which the result is computed.
	 * @return The number of triangles.
	 */
	int numerOfTriangles(int nodeIndex);

	/*!
	 * Clustering coefficient for specific graph.
	 * \note{Validated}
	 *
	 * @return Clustering coefficient.
	 */
	double clusteringCoeficient();

	/*!
	 * Get the nodes degrees of graph
	 * \note{Validated}
	 *
	 * @return Nodes degrees.
	 */
	int* nodeDegrees();

	/*!
	 * Get distance matrix of graph computed by Floyd-Warshall algorithm.
	 * \note{Validated}
	 *
	 * @return Distance matrix
	 */
	SymMatrix<int>* distanceMatrix();

	/*!
	 * Get distance matrix of graph computed by Floyd-Warshall algorithm with additional
	 * path constructions.
	 *
	 * @return Distance matrix
	 */
	SymMatrix<int>* distanceMatrixPaths();

	/*!
	 * Average shortest path in graph determined from distance matrix.
	 * \note{Validated}
	 *
	 * @return Average shortest path.
	 */
	double averageShortestPath();

	/*!
	 * Density for given threshold.
	 * \note{Validated}
	 *
	 * @return Density.
	 */
	double density();

	/*!
	 * Efficiency of graph determined from distance matrix.
	 * \note{Validated}
	 *
	 * @return Efficiency.
	 */
	double efficiency();

	/*!
	 * 	Method to get assortativity coeficient.
	 * \note{Validated}
	 *
	 * @return Assortativity coeficient.
	 */
	double assortativeCoeficient();

	/*!
	 *	Betweeness for given graph.
	 * \note{Validated}
	 *
	 * @return Betweeness.
	 */
	double betweeness();

	double betweenessMP();

	/*!
	 *	Betweeness based on random walk for given graph.
	 *
	 * @return Random walk betweeness.
	 * TODO rethink ALGLIB
	 */
	//double randWalkBetweeness();

	/*!
	 *	Betweeness based on eigenvector for given graph.
	 *
	 * @return Random walk betweeness.
	 * TODO rethink ALGLIB
	 */
	//double eigenvectorBetweeness();

	/*!
	 * 	Alternative algorithm for betweeness computation.
	 *
	 * @return Betweeness.
	 */
	double betweenessParallel(int cpunum);

	/*!
	 * Closeness of graph determined from distance matrix.
	 * \note{Validated}
	 *
	 * @return Closeness centrality.
	 */
	double closeness();

	/*!
	 * Return number of connected components
	 *
	 * @return Number of connected components.
	 */
	int components();

	/*!
	 * 	Method to get nearest neighbors for vertices in graph.
	 *
	 * @return Nearest neighbors array for all vertices
	 */
	double* nodeNearestNeighbors();

	/*!
	 * 	Method to get nearest neighbors for vertices in graph.
	 * 	This method is for reference output and therefore cause the indication of the
	 *  field as non-deletable for descructor
	 *
	 * @return Nearest neighbors array for all vertices
	 */
	double* getNearestNeighbors();

	/*!
	 * 	Method to get betweeness data previously enumerated (if not enumate them)
	 * 	This method is for reference output and therefore cause the indication of the
	 *  field as non-deletable for descructor
	 *
	 * @return Betweeness data for all vertices
	 */
	double* getBetweenessData();

	/*!
	 * 	Method to get random walk betweeness data previously enumerated (if not enumate them)
	 * 	This method is for reference output and therefore cause the indication of the
	 *  field as non-deletable for descructor
	 *
	 * @return Betweeness data for all vertices
	 * TODO rething ALGLIB
	 */
	//double* getRandWalkBetweenessData();

	/*!
	 * 	Method to get eigenvector betweeness data previously enumerated (if not enumate them)
	 * 	This method is for reference output and therefore cause the indication of the
	 *  field as non-deletable for descructor
	 *
	 * @return Betweeness data for all vertices
	 * TODO rething ALGLIB
	 */
	//double* getEigenvectorBetweenessData();

	/*!
	 * 	Method to get closeness data previously enumerated (if not enumate them)
	 * 	This method is for reference output and therefore cause the indication of the
	 *  field as non-deletable for descructor
	 *
	 * @return Closeness data for all vertices
	 */
	double* getClosenessData();

	/*!
	 * 	Method to get Clustering coefficient data previously enumerated (if not enumate them)
	 * 	This method is for reference output and therefore cause the indication of the
	 *  field as non-deletable for descructor
	 *
	 * @return Clustering coefficient data for all vertices
	 */
	double* getClusteringData();

	/*!
	 * 	Method to get Distance matrix data previously enumerated (if not enumate them)
	 * 	This method is for reference output and therefore cause the indication of the
	 *  field as non-deletable for descructor
	 *
	 * @return distance matrix.
	 */
	int* getDistanceMatrixData();

	/*!
	 * 	Method to get nodes degrees data previously enumerated (if not enumate them)
	 * 	This method is for reference output and therefore cause the indication of the
	 *  field as non-deletable for descructor
	 *
	 * @return Node degrees data for all vertices
	 */
	int* getNodeDegrees();

	/**
	 * Generate an Erdos-Renyi graph with probability p = k / (n^2 - n) where k = average degree of actual graph
	 * and n is number of nodes.
	 *
	 * @return Erdos-Renyi graph
	 */
	Graph* sampleErdosRenyi();

	/*!
	 * Small-world index using clustering coefficient C of actual graph and C_er of random graph with average path
	 * length of actual graph L and average path length of random graph L_er. As random graph E-R graph using method
	 * sampleErdosRenyi() is used. Characteristics are combined into:
	 * 	gamma = C / C_er
	 *  lambda = L / L_er
	 * 	S = gamma / lambda
	 *
	 * @return S
	 */
	double smallWorldIndex();

	vector<vector<int> >* getComponentsData();

	double* getComponentDataMat(int &numComp);

private:

	/*! representation of infinity for distances, i.e. rows + 1 */
	int INFTY;
	/*! representation of infinity for sum of all distances, i.e. rows*INFTY */
	int INFTY_SUM;

	/*! adjacency matrix */
	SymMatrix<bool> data;

	/*! Distance matrix */
	SymMatrix<int>* D;

	int** neighs;

	/*! sum of all elements of matrix, i.e. number of edges */
	int numedges;



	/*! Node degrees */
	int* degrees;
	/*! Average degree of nodes */
	double averageDegree;
	/*! Node degrees */
	double* nearestNeighbors;
	/*! clustering coefficient for specific vertices */
	double* clusteringVector;
	/*! betweeness centrality for specific vertices */
	double* eigenvectorBetweenessVector;
	/*! betweeness centrality for specific vertices */
	double* betweenessVector;
	/*! betweeness centrality for specific vertices */
	double* randWalkBetweenessVector;
	/*! closeness for specific vertices */
	double* closenessVector;
	/*! field with predecessors used for Floyd-Warshall*/
	int* predInPath;


	/*! indication that degrees should not be deleted, while they have been getted */
	bool degreesGet;
	/*! indication that nearest neighbors should not be deleted, while they have been getted */
	bool nnGet;
	/*! indication that clustering should not be deleted, while they have been getted */
	bool clusteringGet;
	/*! indication that betweeness should not be deleted, while they have been getted */
	bool betweenessGet;
	/*! indication that random walk betweeness should not be deleted, while they have been getted */
	bool randWalkBetweenessGet;
	/*! indication that random walk betweeness should not be deleted, while they have been getted */
	bool eigenvectorBetweenessGet;
	/*! indication that components should not be deleted, while they have been getted */
	bool componentsGet;
	/*! indication that closeness should not be deleted, while they have been getted */
	bool closenessGet;
	/*! indication that distance matrix should not be deleted, while they have been getted */
	bool dGet;

	vector<vector<int> >* componentsList;
	bool* visited;
	int componentIndex;

	int visit(int i);

	int getComponentMatrices(char*** matrices, int*** indices, int** sizes, int*** subdegrees);

	/**	Method to do initial common settings after constructor
	 * */
	void initialSetttings();

	/**	Sort given array by degrees of nodes, i.e. the length of array is same as number of nodes.
	 *
	 * 	@param array Array that should be sorted.
	 * */
	void sortArrayByDegrees(double* array);

	/**	Shows character of function as assortative or disassortative
	 *
	 * 	@param array Array that should be examined.
	 * 	@return assortative = NN_ASSORTATIVE, disassortative = NN_DISASSORTATIVE and chaotic = NN_CHAOTIC
	 * */
	char checkAssortative(double* array);

	void* betweenessParallelStep(void *args);

	static void* betweenessParallelStepInvoker(void *args);

	struct thread_fun_args
	{
		Graph* This ;
		// other args as required; in this case
		void* actual_arg ;
		thread_fun_args( Graph* t, void* p ) : This(t), actual_arg(p) {}
	};
};

#endif /* CONECTIVITYMATRIX_H_ */
