/*
 * NetworksFunctionsInterface.cpp
 *
 *  Created on: 19.5.2011
 *      Author: hartman
 */

#include "Matrix.h"
#include "Graph.h"
#include "SparseGraph.h"
#include "WeightedGraph.h"


WeightedGraph* loadWeightedGraph(double* mat, int size)
{
	SymMatrix<double>* m = new SymMatrix<double>(mat, size, false, false);
	return new WeightedGraph(m);
}

Graph* loadGraphDist(int* dmat, int size)
{
	SymMatrix<int>* m = new SymMatrix<int>(dmat, size, false, false);
	return new Graph(NULL, m);
}

WeightedGraph* loadWeightedGraphDist(double* dmat, int size)
{
	SymMatrix<double>* m = new SymMatrix<double>(dmat, size, false, false);
	return new WeightedGraph(NULL, m);
}


Graph* loadGraph(double* mat, int size, double threshold)
{
	WeightedGraph* wg = loadWeightedGraph(mat, size);
	Graph* g = wg->getUnweightedGraph(threshold);
	delete wg;
	return g;
}

double* betweenneess_weighted(double* mat, int size)
{
	WeightedGraph* wg = loadWeightedGraph(mat, size);
	double* BC = wg->getBetweenessData();
	delete wg;
	return BC;
}

double* betweenneess_binary(double* mat, int size, double threshold)
{
	Graph* g = loadGraph(mat, size, threshold);
	double* BC = g->getBetweenessData();
	delete g;
	return BC;
}

void betweenneess_binary_sparse(int* mat, int size, double* BC)
{
	SparseGraph* g = new SparseGraph(mat, size);
	g->betweenessTest(BC);
	delete g;
}

double* components_binary(double* mat, int size, double threshold, int &numComp)
{
	Graph* g = loadGraph(mat, size, threshold);
	double* CC = g->getComponentDataMat(numComp);
	delete g;
	return CC;
}

vector<vector<int> >* components_binary_vec(double* mat, int size, double threshold)
{
	Graph* g = loadGraph(mat, size, threshold);
	vector<vector<int> >* CC = g->getComponentsData();
	delete g;
	return CC;
}

double* components_weighted(double* mat, int size, int &numComp)
{
	WeightedGraph* wg = loadWeightedGraph(mat, size);
	double* CC = wg->getComponentDataMat(numComp);
	delete wg;
	return CC;
}

int* degrees_binary(double* mat, double threshold, int size)
{
	Graph* g = loadGraph(mat, size, threshold);
	int* k = g->nodeDegrees();
	delete g;
	return k;
}

double* strengths_weighted(double* mat, int size)
{
	WeightedGraph* wg = loadWeightedGraph(mat, size);
	double* k = wg->strengths();
	delete wg;
	return k;
}

double assortativity_binary(double* mat, int size, double threshold)
{
	Graph* g = loadGraph(mat, size, threshold);
	double r = g->assortativeCoeficient();
	delete g;
	return r;
}

double closeness_binary(double* mat, int size, double threshold)
{
	Graph* g = loadGraph(mat, size, threshold);
	double Cc = g->closeness();
	delete g;
	return Cc;
}

double charpath_len_binary(double* mat, int size, double threshold, int*& D)
{
	Graph* g = loadGraph(mat, size, threshold);
	double Cc = g->averageShortestPath();
	SymMatrix<int>* dmat = g->distanceMatrix();
	D = dmat->getFullData();
	delete dmat;
	delete g;
	return Cc;
}

double charpath_len_weighted(double* mat, int size, double*& D)
{
	WeightedGraph* wg = loadWeightedGraph(mat, size);
	double Cc = wg->averageShortestPath();
	SymMatrix<double>* dmat = wg->distanceMatrix();
	D = dmat->getFullData();
	delete dmat;
	delete wg;
	return Cc;
}

int* distance_mat_binary(double* mat, int size, double threshold)
{
	Graph* g = loadGraph(mat, size, threshold);
	SymMatrix<int>* D = g->distanceMatrix();
	delete g;
	return D->getFullData();
}

double* distance_mat_weighted(double* mat, int size)
{
	WeightedGraph* wg = loadWeightedGraph(mat, size);
	SymMatrix<double>* D = wg->distanceMatrix();
	delete wg;
	return D->getFullData();
}

double charpath_len_dist_weighted(double* dmat, int size)
{
	WeightedGraph* wg = loadWeightedGraphDist(dmat, size);
	double Cc = wg->averageShortestPath();
	delete wg;
	return Cc;
}

double charpath_len_dist_binary(int* dmat, int size)
{
	Graph* g = loadGraphDist(dmat, size);
	double Cc = g->averageShortestPath();
	delete g;
	return Cc;
}

double efficiency_binary(double* mat, int size, double threshold, int*& D)
{
	Graph* g = loadGraph(mat, size, threshold);
	double E = g->efficiency();
	SymMatrix<int>* dmat = g->distanceMatrix();
	D = dmat->getFullData();
	delete dmat;
	delete g;
	return E;
}

double efficiency_weighted(double* mat, int size, double*& D)
{
	WeightedGraph* wg = loadWeightedGraph(mat, size);
	double E = wg->efficiency();
	SymMatrix<double>* dmat = wg->distanceMatrix();
	D = dmat->getFullData();
	delete dmat;
	delete wg;
	return E;
}

double efficiency_dist_weighted(double* dmat, int size)
{
	WeightedGraph* wg = loadWeightedGraphDist(dmat, size);
	double E = wg->efficiency();
	delete wg;
	return E;
}

double efficiency_dist_binary(int* dmat, int size)
{
	Graph* g = loadGraphDist(dmat, size);
	double E = g->efficiency();
	delete g;
	return E;
}

double* delta_betweenness(double* mat, int size, double weightDiffEpsilon)
{
	WeightedGraph* wg = loadWeightedGraph(mat, size);
	wg->setWeightDiffEpsilon(weightDiffEpsilon);
	double* dBC = wg->getDeltaBetweenessData();
	delete wg;
	return dBC;
}

// double* eigvec_betweenness_binary(double* mat, int size, double threshold)
// {
// 	Graph* g = loadGraph(mat, size, threshold);
// 	double* eigBC = g->getEigenvectorBetweenessData();
// 	delete g;
// 	return eigBC;
// }

// double* randwalk_betweenness_binary(double* mat, int size, double threshold)
// {
// 	Graph* g = loadGraph(mat, size, threshold);
// 	double* rwBC = g->getRandWalkBetweenessData();
// 	delete g;
// 	return rwBC;
// }
