/*
 * ConnectivityMatrixSource.h
 *
 *  Created on: 9.10.2009
 *      Author: hartman
 */

#ifndef CONNECTIVITYMATRIXSOURCE_H_
#define CONNECTIVITYMATRIXSOURCE_H_
#include <string>
#include <stack>
#include <queue>
#include <list>
#include "Graph.h"
#include "Matrix.h"

using namespace std;

#define NO_VALUE -1

struct BNode
{
    double dist;
    int predNode;
    int node;
    BNode(double dist, int predNode, int node) :
    	dist(dist),
    	predNode(predNode),
    	node(node)
    {}

    bool operator<(const BNode &n1) const
    {
    	return dist < n1.dist;
    }

    bool operator>(const BNode &n1) const
    {
    	return dist > n1.dist;
    }
};

struct ASPData
{
    int s;
    int t;
    int u;
    double w;

    vector<int> *P;
    long *sigma;
    bool *nu;
    long *sigma_T;

    ASPData(int s, int t, int u, double w, long *sigma, bool *nu, vector<int> *P, long *sigma_T) :
    	s(s),t(t),u(u),w(w),sigma(sigma),nu(nu),P(P),sigma_T(sigma_T)
    {}

    ASPData *clone(int au, double aw)
    {
    	ASPData *nd = new ASPData(s,t,au,aw,sigma,nu,P,sigma_T);
    	return nd;
    }
};

class WeightedGraph {
public:
	/**	Constructor
	 *
	 * 	@param adjMat Adjacency matrix that initialize a graph
	 * 	@param symmetric if only symmetric part of matrix should be loaded (upper triangular)
	 * 	@param absolute if values should be taken absolute
	 * */
	WeightedGraph(SymMatrix<double>* adjMat);

	WeightedGraph(SymMatrix<double>* mat,SymMatrix<double>* dmat);

	/**	Destructor
	 * */
	virtual ~WeightedGraph();

	/*! Initial settings run from constructor. It prepares some basic data structures.
	 */
	void initialSetttings();

	vector<double> getEdges();


	double efficiency();

	/**	Get the corresponding unweighted matrix applying given threshold. Diagonal elements are automatically set to zero.
	 *
	 * @return Unweighted matrix
	 * */
	Graph* getUnweightedGraph(double threshold);

	/*!
	 * Get size of matrix which means number of its rows.
	 *
	 * \note{Validated}
	 */
	int getSize();

	/*!
	 * 	Method to get number of edges of ghraph
	 * \note{Validated}
	 *
	 * @return Number of edges.
	 */
	int getNumberOfEdges();

	/*!
	 * Method to get number of edges of graph when threshold applied
	 * \note{Validated}
	 *
	 * @param threshold
	 * @return Number of edges when given threshold applied
	 */
	int getNumberOfEdges(double threshold);

	/*!
	 * 	Method to get sum of all weigths
	 *
	 * @return Sum of all weigths for all edges.
	 */
	double sumOfWeigths();

	/*!
	 * 	Method to get density of graph which means sum of its weightd divided by number of edges
	 *
	 * @return Density of graph.
	 */
	double getDensity();

	/*!
	 * Perform cut opera
	 * @return
	 */
	bool cut();


	vector<double>* getThresholds(vector<double>* densities);


	/*!
	 * 	Method to get thresholds that should be used to obtain all possible graph densities.
	 *
	 *	@param t0 threshold from which we should start
	 *	@param thresholds resulting thresholds
	 *	@param densities resulting densities
	 *	@param maxTresholdIndex Index of the threshold after which there are same densities
	 *	@return resulting size of array
	 */
	//int getThresholds(double t0, double* &thresholds, double* &densities, int *maxTresholdIndex);

	/*!
	 * Method to find new minimum value stored in adjacency matrix sharply larger than given value
	 *
	 * @param value Bounding value
	 * @return minimum from adjacency matrix sharply larger than value
	 */
	double findNextMin(double value);

	/*!
	 * 	Return unweighted density if the only used choice is zero and nonzero
	 *
	 * @return Unweighted density of graph
	 */
	double getUnweightedDensity();

	/*!
	 * Method to get weighted density of graph when threshold is applied
	 *
	 * @param threshold Threshold applied to graph resulting in undirected with specific density
	 * @return Unweighted density of graph when applying given threshold
	 */
	double getUnweightedDensity(double threshold);


	/**	Method to generate random matrix and store into file
	 *
	 * 	@param filename file into which it is saved
	 * 	@param size size of generated matrix
	 * 	@param min minimum value in matrix
	 *  @param max maximum value in matrix
	 *
	 *	@return 0 if everything is O.K. and -1 if file cannot be opened.
	 * */
	static int generateRandomMatrix(string filename, int size, double min, double max);

	/** Get value from given position
	 *
	 * 	@param i row specification
	 * 	@param j column specification
	 *  @return value from position i,j
	 * */
	double operator() (int i, int j);

	/** Get distance between two vertices if enumerated.
	 *
	 * 	@param i first vertex
	 * 	@param j second vertex
	 *  @return distance between two vertices if enumerated, othewise -1.
	 * */
	double dist(int i, int j);

	/** Enumerate strength of nodes
	 *
	 * @return strengths array
	 * */
	double* strengths();

	/*!
	 * Enumerate average strength of nodes
	 *
	 * @return average strength of nodes
	 * */
	double averageStrength();

	/**	Minimum of values in matrix
	 * */
	double min();

	/**	Minimum of values in matrix
	 * */
	double max();

	/*!
	 * Clustering coefficient for specific graph.
	 * \note{Validated}
	 *
	 * @return Clustering coefficient.
	 */
	double clusteringCoefficient();

	double* getClusteringCoefficientData();

	/*!
	 * Return number of connected components
	 *
	 * @return Number of connected components.
	 */
	int components();

	/*!
	 *	Betweeness for given graph.
	 * \note{Validated}
	 *
	 * @return Betweeness.
	 */
	long double betweeness();

	/*!
	 *	\delta-Betweeness for given graph.
	 *
	 * @return Betweeness.
	 */
	double deltaBetweeness();


	/*!
	 * 	Method to get betweeness data previously enumerated (if not enumate them)
	 * 	This method is for reference output and therefore cause the indication of the
	 *  field as non-deletable for descructor
	 *
	 * @return Betweeness data for all vertices
	 */
	double* getBetweenessData();

	double* getDeltaBetweenessData();

	/*!
	 * Get distance matrix of graph computed by Floyd-Warshall algorithm.
	 *
	 * @return Distance matrix
	 */
	SymMatrix<double>* distanceMatrix();

	double averageShortestPath();

	vector<BNode>* getPath(vector<BNode> *P, int i, int j);

	void resetValues();

	int numerOfTriangles(int nodeIndex);

	void printMatrix(int max);
	void printMatrix(double* A, int max);

	void setWeightDiffEpsilon(double aWeightDiffEpsilon);

	vector<vector<int> >* getComponentsData();

	double* getComponentDataMat(int &numComp);


private:
	/** Data - triangular matrix */
	SymMatrix<double> data;

	/// ------------- Some auxiliary characteristics ------------------
	/*! sum of all elements of matrix, i.e. number of edges */
	int numedges;
	/*! reference to number with sum of all weights - possibly NULL */
	double *sumweights;

	/// ------------- numerical parameters ------------------
	/*! Possible difference of paths */
	double weightDiffEpsilon;
	/*! Numeric precision definition */
	double precisionEpsilon;

	/// ------------- network characteristics ------------------
	/*! density of graph or < NO_VALUE if not initiated */
	double density;
	/** strengths for all nodes - each one is a sum of all weights for all edges adjacent to corresponding node */
	double* strengthsData;

	/*! indication that components should not be deleted, while they have been getted */
	bool componentsGet;
	/*! Components - first dimension is components and second is nodes in these components */
	vector<vector<int> >* componentsList;
	bool* visited;
	int componentIndex;

	/// TODO
	/*! clustering coefficient for specific vertices */
	double* clusteringVector;
	/*! betweeness centrality for specific vertices */
	double* betweenessVector;
	/*! betweeness centrality for specific vertices */
	double* deltaBetweenessVector;

	/// ------------- indicators for network characteristics ------------------

	bool strengthsGet;
	bool betweenessGet;
	bool deltaBetweenessGet;

	/*! Distance matrix */
	double infinite;
	//double* D;
	SymMatrix<double>* D;
	int* next;

	/**	Auxiliary method printing whole matrix (only for first max x max block)
	 *
	 * 	@param max maximal value for index.
	 * */

	int accessTriu(int *A, int i, int j);

	int visit(int i);

	void printTriangular();

	void absoluteValues();

	void outSpace(int k);

	void ASP(ASPData *adata, int num);

	bool findNextMin(double value, double *restrictedMin);


};

#endif /* CONNECTIVITYMATRIXSOURCE_H_ */
